$TITLE	MPSGE2MCP - A SAMPLE BILATERAL TRADE MODEL FOR THE WORLD ECONOMY BASED ON GTAP4E-DATA

*	Include data based on GTAP4E at the highest level of regional and sectoral disaggregation available
$LIBINCLUDE mrtdata GTAP4E


*	Drop "zero" values (1e-6) for va which might have been left from some
*	lower bound operations in the GTAP4E synthesis process
va(d,i,r)$(va(d,i,r) le 1e-6) = 0;



SET	abate(r)	Regions with official abatement target;
abate(r)   = no;


*	Define subset for more sophisticated nesting of energy demand (distinction between ELE and other sectors)
*	and distinction of exhaustible energy supply sectors (XE) 

SETS
	e(i)	Energy goods			/col, oil, cru, gas, ele/,
	ec(i)	Fossil fuels and electrictity	/col, oil, gas, ele/,
	fe(i)	Fossil fuels			/col, oil, gas/
	ele(i)	Electricity			/ele/,
	xe(i)	Exhaustible energy		/cru,gas,col/,
	cru(i)	Crude oil			/cru/,
	oil(i)	Oil				/oil/,
	gas(i)	Natural gas			/gas/,
	coal(i)	Coal				/col/,
	ne(i)	Non energy goods		
	nrp(i)	Non resource production sectors 

	capital(f)				/cap/;

ne(i)$(not e(i))	= yes;
nrp(i)$(not xe(i))	= yes;


PARAMETER	gsize(r)	Size of public provision;
gsize(r) = 1;

PARAMETERS
	pa0(j,i,r)	Reference price for Armington inputs into production,
	transfer(r)	Lump sum transfer between RA and GOVT to balance the budgets;
pa0(j,i,r)  = 1 + ti(j,i,r);

*	At this point we might respecify the model to a bilateral trade setting with a SOE closure
*	towards the Rest of the World
*	In the default we use a full world trade model (the flag SOE is set to 0)
SCALAR	SOE soe setting /0/;

SET	re(r)	Regions which are part of the explicit bilateral trade model 
	/	
 AUS    Australia
 NZL    New Zealand
 JPN    Japan
 KOR    Republic of Korea
 IDN    Indonesia
 MYS    Malaysia
 PHL    Philippines
 SGP    Singapore
 THA    Thailand
 VNM	Vietnam
 CHN    China
 HKG    Hong Kong
 TWN    Taiwan
 IND    India
 LKA	Sri Lanka
 RAS    Rest of South Asia
 CAN    Canada
 USA    United States of America
 MEX    Mexico
 CAM    Central America and Caribbean
 VEN	Venezuela
 COL	Columbia
 RAP	Rest of Andean Pact
 ARG    Argentina
 BRA    Brazil
 CHL    Chile
 URY	Uruguay
 RSM    Rest of South America
 GBR	United Kingdom
 DEU	Germany
 DNK	Denmark
 SWE	Sweden
 FIN	Finland
 REU	Rest of EU,
 EFT    European Free Trade Area
 CEA    Central European Associates
 FSU    Former Soviet Union
 TUR	Turkey
 RME	Rest of Middle East
 MAR	Morocco
 RNF	Rest of North Africa
 SAF	South Africa
 RSA	Rest of South Africa
 RSS	Rest of South-Saharan Africa
 ROW    Rest of World  
	/;


ALIAS (re,se);

*	If SET re is only a subset of all regions r then adopt the SOE setting
*	and represent all regions being not part of re via aggregate ROW export supplies and import demands 
soe$(CARD(re) EQ CARD(r)) = 0;
DISPLAY soe;

PARAMETERS
	xrow		Exports to ROW
	mrow		Imports from ROW
	pmx0_row	Reference price for ROW imports
	vtwr_row	Value of transport services associated with imports from ROW
	tm_row		Tariff rate on ROW imports ;

DISPLAY vxm;
xrow(i,re)	= 0;
xrow(i,re)$soe	= sum(r$(not re(r)), vxmd(i,re,r)) + vst(i,re);
vom(i,re)	= vom(i,re) - xrow(i,re);
DISPLAY xrow, vom;

*	Note: pmx0(i,s,r) = (1+ TX(i,s,r) +(tm(i,s,r)*(1+tx(i,s,r))))
mrow(i,re)	= sum(r$(not se(r)), vxmd(i,r,re)*(1+TX(i,r,re)));
vtwr_row(i,re)	= sum(r$(not se(r)), vtwr(i,r,re));
tm_row(i,re)$mrow(i,re) = [sum(r$(not se(r)), tm(i,r,re)*(vxmd(i,r,re)*(1+TX(i,r,re))))
			    + sum(r$(not se(r)), vtwr_row(i,re)*tm(i,r,re))]	
		    /(mrow(i,re) + vtwr_row(i,re));
pmx0_row(i,re)	= 1 + tm_row(i,re);

*	Adjust government budgets for tariff revenues from exports to ROW in the BMK
PARAMETER	tax_row	BMK tariff revenues of country re from exports to ROW;
tax_row(re) = sum(r$(not re(r)), sum(i, vxmd(i,re,r)*tx(i,re,r)));

*	Compute lump-sum transfer to balance budgets
transfer(r) = sum(f, evoa(f,r))+ vb(r) -vi(r) -	 vp(r);
DISPLAY transfer;


PARAMETERS
	esub(r)		Elasticity between VA and energy
	etrn(r)		Elasticity of transformation
	e_vae(r)	Elasticity of substitution in the VA production nest
	e_elec(r)	Elasticity between electric and non-electric energy
	e_c_go(i,r)	Elasticity between coal and the oil-gas aggregate 
	e_g_o(i,r)	Elasticity of substitution between oil and gas in production
	esub_es(i,r)	Elasticity of substitution in energy production
	thetar(i,r)	Value share of resource inputs in fossil supply
	eta_g(r)	Elasticity of substitution between fossil fuels in final demand
	eta_c(r)	Elasticity of substitution between energy and non energy composite in final demand
	eta(xe)		Elasticity of supply 
	esubdm(i,r)	Armington elasticity - domestic vesus imports
	esubmm(i,r)	Armington elasticity - intra-imports ;

esub(r)		= 0.5;
etrn(r)		= 2;
e_vae(r)	= 1.0;
e_elec(r)	= 0.1;
e_c_go(i,r)	= 0.5;
e_g_o(i,r)	= 2.0;
eta_g(r)	= 0.3;
eta_c(r)	= 0.5; 
eta("cru")	= 1.0;
eta("gas")	= 1.0;
eta("col")	= 0.5;
esubdm(i,r)	= 4;
esubmm(i,r)	= 8;
esubdm(fe,r)	= 0.5;

*	Establish resource inputs for resource supply:
PARAMETERS
	rd0(i,r)	Resource inputs for fossil fuel production
	prref(i,r)	Reference price of energy-supply resource
	ffsupply(i,r)	Fossil fuel supply;

rd0(xe,r)	= min(vfm("CAP",xe,r), 0.5 * vom(xe,r) * (1-ty(xe,r))) ;
vfm("CAP",xe,r) = round(vfm("CAP",xe,r) - rd0(xe,r), 6);
evoa("CAP",r)	= sum(i, vfm("CAP",i,r));
esub_es(xe,r)	= 0;
prref(xe,r)	= 1;
ffsupply(xe,r)	= 1$rd0(xe,r);
DISPLAY rd0, vfm;

thetar(xe,r)$vom(xe,R) = rd0(xe,r) / ( (vom(xe,r)+xrow(xe,r)) * (1-ty(xe,r)));
*	With respect to MCP formulation assure CES function
esub_es(xe,R)$(1-thetar(xe,r)) = thetar(xe,r) * ETA(xe) / (1-thetar(xe,r)) + 0.00001;
DISPLAY	 thetar, esub_es;

*	======================================================================
*	Generate base year carbon emissions by sector and then echo-print some
*	benchmark statistics:

PARAMETERS
	carblim(r)	Total regional emission budget
	carbon95(r,*)	1995 carbon emissions (IEA)
	carbcoef(j,*,r) Carbon coefficients
	co2punit(j,*,r)
	cecphys(*)	carbon emission coefficients for physical units 
				/col 0.024, gas 0.0137, oil 0.0181 /;

carbcoef(j,d,r)		= 0;
carbcoef(fe,"final",r)	= cecphys(fe) * efd(fe,r);
carbcoef(fe,i,r)	= cecphys(fe) * eind(fe,i,r);
carbon95(r,fe)		= carbcoef(fe,"final",r) + sum(i, carbcoef(fe,i,r));
carbon95(r,"total")	= sum(fe, carbon95(r,fe));
carblim(r)		= sum(fe, carbcoef(fe,"final",r)+sum(i,carbcoef(fe,i,r)));
DISPLAY carblim, carbon95;

*	Final demand in GTAP-E includes provision of public goods
*	For this equal-yield analysis we want to treat government demand separately
*	and split up final demand proportionally into government demand and household demand 
carbcoef(fe,"hh",r)$(vpm(fe,r) + vgm(fe,r)) = carbcoef(fe,"final",r)* vpm(fe,r)/(vpm(fe,r) + vgm(fe,r));
carbcoef(fe,"govt",r)$(vpm(fe,r) + vgm(fe,r)) = carbcoef(fe,"final",r) - carbcoef(fe,"hh",r);
co2punit(j,i,r) = 0;
co2punit(fe,i,r)$vafm(fe,i,r)	= carbcoef(fe,i,r)/vafm(fe,i,r);
co2punit(fe,"final",r)$vpm(fe,r)= carbcoef(fe,"final",r)/vpm(fe,r); 
co2punit(fe,"govt",r)$vgm(fe,r) = carbcoef(fe,"govt",r)/vgm(fe,r);
display co2punit;

PARAMETERS
	tpunit(i,r,s)	    Unit transport cost coefficient;
tpunit(i,r,s)$vxmd(i,r,s) = vtwr(i,r,s) / vxmd(i,r,s);

ALIAS(e,ee);
ALIAS(f,ff);
ALIAS(fe,ffe);
ALIAS(se,sse);
ALIAS(j,jj);

$ONTEXT

$MODEL:GTAP

$SECTORS:
	C(re)			! Private consumption
	G(re)			! Public provision
	Y(i,re)$vom(i,re)	! Output
	M(i,re)$vim(i,re)	! Import aggregation
	A(d,i,re)$va(d,i,re)	! Armington aggregation of domestic and imports
	YT$(not soe)		! Transport 
	LS(re)			! Labor supply activity   

$COMMODITIES:
	PC(re)			! Private demand
	PG(re)			! Public provision
	PY(i,re)$(vom(i,re))	! Output price
	PM(i,re)$vim(i,re)	! Import price
	PA(d,i,re)$va(d,i,re)	! Armington composite price
	PF(f,re)$evoa(f,re)	! Factor price
	PLS(re)			! Price of labor
	PT$(not soe)		! Transport services
	PFX$soe			! Price of foreign exchange 
	PCARB(re)$abate(re)	! Price of carbon
	PR(i,re)$rd0(i,re)	! Energy resource


$CONSUMERS:
	RA(re)			! Representative agent
	GOVT(re)		! Government 

$AUXILIARY:
	TAU(re)			! Endogenous equal yield tax adjusmtent

*	Armington aggregation over domestic versus imports:

$PROD:A(d,i,re)$va(d,i,re)  S:esubdm(i,re)
	O:PA(d,i,re)	Q:va(d,i,re) 
	I:PY(i,re)	Q:vd(d,i,re)
	I:PM(i,re)	Q:vm(d,i,re)

 
*	Domestic production

$PROD:Y(i,re)$((vom(i,re)>0)$nrp(i)) T:etrn(re) S:0  vae(s):esub(re)	va(vae):e_vae(re) 
+ e(vae):e_elec(re)  nel(e):e_c_go(i,re) lqd(nel):e_g_o(i,re) 
+ oil(lqd):0 col(nel):0 gas(lqd):0 cru(s):0 
	O:PY(i,re)			Q:vom(i,re)				A:GOVT(re) T:ty(i,re)
	O:PFX$soe			Q:xrow(i,re)				A:GOVT(re) T:ty(i,re)
	I:PA("i",j,re)			Q:vafm(j,i,re)		P:pa0(j,i,re)	A:GOVT(re) T:ti(j,i,re) e:$ele(j) j.tl:$fe(j) 
	I:PCARB(re)#(fe)$abate(re)	Q:carbcoef(fe,i,re)	P:1.e-6						  fe.tl:
	I:PF(f,re)			Q:vfm(f,i,re)		P:pf0(f,i,re)	A:GOVT(re) T:tf(f,i,re) va:


*	Fossil fuel production activity (crude, gas and coal):

$PROD:Y(xe,re)$vom(xe,re)  T:etrn(re)	S:esub_es(xe,re)  id:0 
	O:PY(xe,re)			Q:vom(xe,re)		A:GOVT(re) T:ty(xe,re) 
	O:PFX$soe			Q:xrow(xe,re)		A:GOVT(re) T:ty(xe,re) 
	I:PA("i",j,re)			Q:vafm(j,xe,re)		P:pa0(j,xe,re)	A:GOVT(re) T:ti(j,xe,re)	id:
	I:PF(f,re)			Q:vfm(f,xe,re)		P:pf0(f,xe,re)	A:GOVT(re) T:tf(f,xe,re)	id:
	I:PCARB(re)#(fe)$abate(re)	Q:carbcoef(fe,xe,re)	P:1.e-6						id:  
	I:PR(xe,re)			Q:rd0(xe,re)		P:prref(xe,re)		 

* Import demand

$PROD:M(i,re)$(vim(i,re)>0)   S:esubmm(i,re)   se.TL:0 
	O:PM(i,re)		Q:vim(i,re)
	I:PY(i,se)		Q:vxmd(i,se,re)			P:pmx0(i,se,re)	se.TL:
+		A:GOVT(se) T:TX(i,se,re) A:GOVT(re) T:(tm(i,se,re)*(1+tx(i,se,re)))
	I:PT#(se)$(not soe)	Q:vtwr(i,se,re)			P:pmt0(i,se,re)	se.TL: 
+		A:GOVT(re) T:tm(i,se,re)
	I:PFX#(se)$soe		Q:vtwr(i,se,re)			P:pmt0(i,se,re)	se.TL: 
+		A:GOVT(re) T:tm(i,se,re)
	I:PFX$soe		Q:(mrow(i,re)+ vtwr_row(i,re))	P:pmx0_row(i,re)	
+		A:GOVT(re) T:tm_row(i,re) 


* Labor supply

$PROD:LS(re)
       O:PF("LAB",re)		Q:evoa("LAB",re)
       I:PLS(re)		Q:evoa("LAB",re)	A:GOVT(re) N:tau(re) 


*	Demand for public output:

$PROD:G(re)  S:1 e:eta_g(re) gas(e):0 col(e):0 oil(e):0
	O:PG(re)			Q:vg(re)
	I:PA("g",i,re)			Q:vgm(i,re)			P:pg0(i,re)	A:GOVT(re) T:tg(i,re)	i.tl:$fe(i)
	I:PCARB(re)#(fe)$abate(re)	Q:carbcoef(fe,"govt",re)	P:1.e-6					fe.tl:


*	Private consumption:

$PROD:C(re)  s:eta_c(re) c:1 e:1  oil(e):0 col(e):0 gas(e):0
	O:PC(re)			Q:vp(re)
	I:PA("c",i,re)			Q:vpm(i,re)		P:pc0(i,re)	A:GOVT(re) T:tp(i,re) 
+		i.tl:$fe(i) c:$(not e(i)) e:$ele(i) 
	I:PCARB(re)#(fe)$abate(re)	Q:carbcoef(fe,"hh",re)	P:1.e-6 fe.tl:


*	Inter-national transport services (Cobb-Douglas):

$PROD:YT$(not soe)  S:1
	O:PT		Q:vt
	I:PY(i,re)	Q:vst(i,re)
    

*	Final demand over consumption, savings and government 
*	services (Cobb-Douglas):

$DEMAND:RA(re)	
	E:PR(i,re)		Q:(rd0(i,re) *ffsupply(i, re)) 
	E:PF("cap",re)		Q:evoa("CAP",re)
	E:PLS(re)		Q:evoa("LAB",re)	
	E:PC(num)$(not soe)	Q:vb(re)
	E:PFX$soe		Q:vb(re)
	E:PY(cgd,re)		Q:(-vi(re))
	E:PC(re)		Q:(-transfer(re))	
	D:PC(re)		Q:vp(re)


*	Government

$DEMAND:GOVT(re)
	E:PCARB(re)$abate(re)	Q:carblim(re)
	E:PC(re)		Q:transfer(re)	      
	E:PFX$soe		Q:tax_row(re)
	D:PG(re)		Q:vg(re)


*	Equal yield constraint - yields tax rate multiplier

$CONSTRAINT:TAU(RE)
	G(re) =E= GSIZE(re);


$OFFTEXT
$SYSINCLUDE mpsgeset GTAP


*   Check the benchmark:
TAU.FX(re)	    = 0;
PCARB.L(re)	    = 1.e-6;


GTAP.WORKSPACE = 32;
GTAP.ITERLIM = 0;
$INCLUDE GTAP.GEN
SOLVE GTAP USING MCP;
ABORT$(ABS(gtap.OBJVAL) gt 1e-4) "Model does not calibrate", gtap.OBJVAL;
DISPLAY "The following reports precision of the benchmark dataset:", gtap.OBJVAL;


*   Formulate the model in MCP
*   ============================================
 
*   Define value shares to write down functional forms in share form
 
PARAMETERS
	thetag(*,r)		Share of Armington inputs in public good production
	vs_fe_e_g(fe,r)		Share of fossil fuel in total fe input of public good production
	thetac_e(r)		Share of fossil and energy input in total final demand
	vs_i_ne_c(i,r)		Share of good i in non-energy aggregate of final demand
	vs_e_e_c(e,r)		Share of fuel in energy aggregate of final demand
	thetaa_m(d,i,r)		Import value share in Armington production
	vs_y_t(i,r)		Share of good i by region r into transport service
	vs_f_va(f,i,r)		Share of factor f in value added of sectoral production
	vs_yt_m(i,s,r)		Value share of YT aggregate in import
	vs_y_yt(i,s,r)		Value share of Y in YT aggregate in import
	vs_ypfx_m(i,s,r)	Value share of YPFX aggregate in import
	vs_y_ypfx(i,s,r)	Value shre of Y in YPFX aggregate in import
	vs_pfx_m(i,r)		Value share of PFX aggregate in import
	v_f_nrp(f,i,r)		Share of factor f in value added of nr production
	v_kle_nrp(i,r)		Value share of KLE aggregate in total nr production
	v_ne_nrp(i,i,r)		Value share of non-energy armington goods in total nr production
	v_cru_nrp(i,r)		Value share of cru in total nr production
	v_va_nrp(i,r)		Value share of VA in KLE aggregate
	v_ele_nrp(i,r)		Value share of ELE in energy aggregate
	v_col_nrp(i,r)		Value share of COA in non-ele energy aggregate
	v_gas_nrp(i,r)		Value share of GAS in gas-oil energy aggregate
	vs_r_xe(xe,r)		Value share of resource in fossil fuel production
	vs_f_xe(f,xe,r)		Value share of factor input in fossil fuel production
	vs_a_xe(j,xe,r)		Value share of Armington input in fossil fuel production
	vs_m_y(j,i,r)		Value share of non-energy Armington input into production
	thetad(i,r)		Value share of production destined for the domestic market (SOE case)
	thetat(i,r)		Value share of domestic input into international transport services
	adj_base(i,se,re)	;

 
vs_m_y(j,i,r)$vafm(j,i,r)				= vafm(j,i,r)*pa0(j,i,r)/(vom(i,r)+xrow(i,r)$soe);
thetag(i,r)$((not fe(i))$vg(r))				= pg0(i,r) * vgm(i,r) / vg(r);
thetag("fe",r)$vg(r)					= sum(i$fe(i), pg0(i,r) * vgm(i,r)) / vg(r);
vs_fe_e_g(fe,r)$sum(ffe, pg0(ffe,r) * vgm(ffe,r))	= pg0(fe,r) * vgm(fe,r)/ sum(ffe, pg0(ffe,r) * vgm(ffe,r));
thetac_e(r)$vp(r)					= (sum(i$fe(i), pc0(i,r) * vpm(i,r)) + pc0("ele",r)*vpm("ele",r))
								/ vp(r);
vs_i_ne_c(i,r)$(sum(j$(not ec(j)), pc0(j,r) * vpm(j,r)))
							= (pc0(i,r) * vpm(i,r))
								/ sum(j$(not ec(j)), pc0(j,r) * vpm(j,r));
vs_e_e_c(e,r)$(sum(ee, pc0(ee,r) * vpm(ee,r)))		= (pc0(e,r) * vpm(e,r))
								/ (sum(ee, pc0(ee,r) * vpm(ee,r)));
thetaa_m(d,i,r)$va(d,i,r)				= vm(d,i,r)/va(d,i,r);
vs_y_t(i,r)$vt						= vst(i,r) /vt;
vs_yt_m(i,s,r)$vim(i,r)					= (vxmd(i,s,r)*pmx0(i,s,r)+ vtwr(i,s,r)*pmt0(i,s,r))/vim(i,r);
vs_y_yt(i,s,r)$(vxmd(i,s,r)*pmx0(i,s,r)+ vtwr(i,s,r)*pmt0(i,s,r)) 
							=  vxmd(i,s,r)*pmx0(i,s,r) / (vxmd(i,s,r)*pmx0(i,s,r)+ vtwr(i,s,r)*pmt0(i,s,r));
vs_ypfx_m(i,s,r)$vim(i,r)				= (vxmd(i,s,r)*pmx0(i,s,r)+vtwr(i,s,r)*pmt0(i,s,r))/vim(i,r);
vs_pfx_m(i,r)$vim(i,r)					= (mrow(i,r)+ vtwr_row(i,r))*pmx0_row(i,r)/vim(i,r);
vs_y_ypfx(i,s,r)$(vxmd(i,s,r)*pmx0(i,s,r)+vtwr(i,s,r)*pmt0(i,s,r)) 
							= vxmd(i,s,r)*pmx0(i,s,r) / (vxmd(i,s,r)*pmx0(i,s,r) + vtwr(i,s,r)*pmt0(i,s,r));
vs_f_va(f,i,r)$(sum(ff,vfm(ff,i,r)*pf0(ff,i,r)))	= (vfm(f,i,r)*pf0(f,i,r))/sum(ff,vfm(ff,i,r)*pf0(ff,i,r));
v_f_nrp(f,i,r)$(sum(ff, (vfm(ff,i,r)*pf0(ff,i,r))))    
							= (vfm(f,i,r)*pf0(f,i,r))/
								(sum(ff, (vfm(ff,i,r)*pf0(ff,i,r))));
 v_kle_nrp(i,r)$(vom(i,r))				= (sum(f, vfm(f,i,r)*pf0(f,i,r))
								+ sum(j$(fe(j) or ele(j)), vafm(j,i,r)*pa0(j,i,r))) 
								/((vom(i,r)+xrow(i,r))*(1-ty(i,r)));
v_ne_nrp(i,i,r)$(vom(i,r))				= (vafm(i,i,r)*pa0(i,i,r)) /((vom(i,r)+xrow(i,r))*(1-ty(i,r)));
v_cru_nrp(i,r)$(vom(i,r))				= (vafm("cru",i,r)*pa0("cru",i,r)) /((vom(i,r)+xrow(i,r))*(1-ty(i,r)));
v_va_nrp(i,r)$(sum(f,vfm(f,i,r)*pf0(f,i,r)) + sum(j$(fe(j) or ele(j)), vafm(j,i,r)*pa0(j,i,r)))
							= sum(f,vfm(f,i,r)*pf0(f,i,r))/( sum(f,vfm(f,i,r)*pf0(f,i,r))
								+ sum(j$(fe(j) or ele(j)), vafm(j,i,r)*pa0(j,i,r)));
v_ele_nrp(i,r)$(sum(j$(fe(j) or ele(j)), vafm(j,i,r)*pa0(j,i,r)))
							= (vafm("ele",i,r)*pa0("ele",i,r))/
								sum(j$(fe(j) or ele(j)), vafm(j,i,r)*pa0(j,i,r));
v_col_nrp(i,r)$(sum(j$fe(j), vafm(j,i,r)*pa0(j,i,r)))
							= (vafm("col",i,r)*pa0("col",i,r))/
								sum(j$fe(j), vafm(j,i,r)*pa0(j,i,r));
v_gas_nrp(i,r)$(vafm("gas",i,r)*pa0("gas",i,r)+ vafm("oil",i,r)*pa0("oil",i,r))
						= (vafm("gas",i,r)*pa0("gas",i,r))/
								(vafm("gas",i,r)*pa0("gas",i,r)+ vafm("oil",i,r)*pa0("oil",i,r));
vs_r_xe(xe,r)$(vom(xe,r)+xrow(xe,r))			= (rd0(xe,r)*prref(xe,r))/
								((vom(xe,r)+xrow(xe,r))*(1-ty(xe,r)));
vs_f_xe(f,xe,r)$vom(xe,r)				= (vfm(f,xe,r)*pf0(f,xe,r))/
								(sum(ff, (vfm(ff,xe,r)*pf0(ff,xe,r)))+
								sum(j, vafm(j,xe,r)*pa0(j,xe,r)));
vs_a_xe(j,xe,r)$vom(xe,r)				= (vafm(j,xe,r)*pa0(j,xe,r))/
								(sum(f, (vfm(f,xe,r)*pf0(f,xe,r)))+
								sum(jj, vafm(jj,xe,r)*pa0(jj,xe,r)));
thetad(i,r)$SOE						= vom(i,r)/(vom(i,r) + xrow(i,r));
thetat(i,r)						= vst(i,r)/vt;

*	Armington aggregation across imports from different countries:
*	Incorporate a multiplier for PFX or PT in Leontief nest with PY because A_M_Y refers to VXMD basis
adj_base(i,se,re)$vxmd(i,se,re)				= vtwr(i,se,re)/vxmd(i,se,re);

 
*	Declare subsets of d in order to make it possible to write equation MKT_PA:
SETS	c_d(d)/c/,  g_d(d)/g/, i_d(d) /i/;

VARIABLES
	A_G(i,r)	Public sector unit demand
	A_C(i,r)	Private unit demand
	A_A_Y(d,i,r)	Demand for domestic input into Armington production
	A_A_M(d,i,r)	Demand for import composite into Armington production
	A_M_Y(i,r,s)	Import unit demand from region r to region s
	A_T(i,r)	Transport services
	A_M_PFX(i,r)	ROW import demand
	A_F_NR(f,i,r)	Factor demand in non-resource production
	A_F_R(f,i,r)	Factor demand in fossil fuel production
	A_A_NR(j,i,r)	Armington good demand in non-fossil production
	A_A_R(j,i,r)	Armington demand in fossil fuel production
	A_R_R(i,r)	Resource demand in fossil fuel production
	A_SD_Y(i,r)	Supply of production to domestic market
	A_SX_Y(i,r)	Supply of production to ROW (PFX)
	A_P_KLE(i,r)	kle price aggregate in nrp
	A_P_LIQ(i,r)	liq price aggregate
	A_P_FF(i,r)	ff price aggregate
	A_P_EN(i,r)	Energy price aggregate
	A_P_VA(i,r)	va price aggregate;
 
EQUATIONS
*   Compensated unit demand and supply functions
	DEF_G(i,r)	Public sector demand for Armington good
	DEF_C(i,r)	Private demand for Armington good
	DEF_A_Y(d,i,r)	Demand for domestic good in Armington production
	DEF_A_M(d,i,r)	Demand for import composite in Armington production
	DEF_T(i,r)	Inter-national transport services (Cobb-Douglas)
	DEF_M_Y(i,s,r)	Import demand from region re for good from region se
	DEF_M_PFX(i,r)	ROW import demand from region re
	DEF_F_NR(f,i,r)	Factor demand in non-resource production
	DEF_F_R(f,i,r)	Factor demand in fossil fuel production
	DEF_A_NR(j,i,r)	Armington demand in non-fossil production
	DEF_A_R(j,i,r)	Armington demand in fossil fuel production
	DEF_R_R(i,r)	Resource demand in fossil fuel production
	DEF_SD_Y(i,r)	Supply of production to domestic market
	DEF_SX_Y(i,r)	Supply of production to ROW (PFX)
	DEF_P_LIQ(i,r)	LIQ price aggregate
	DEF_P_FF(i,r)	FF price aggregate
	DEF_P_EN(i,r)	Energy price aggregate
	DEF_P_VA(i,r)	VA price aggregate
	DEF_P_KLE(i,r)	KLE price aggregate
 
*   Zero-Profit conditions
	PRF_C(r)	Zero profit for private consumption
	PRF_G(r)	Zero profit for public provision
	PRF_Y(i,r)	Zero profit for aggregate output
	PRF_M(i,r)	Zero profit for import aggregation
	PRF_A(D,i,r)    Zero profit for Armington aggregation of domestic and imports
	PRF_YT		Zero profit for transport
	PRF_LS(r)	Zero profit for labor supply

*   Market clearance condition
	MKT_PC(r)	Private demand
	MKT_PG(r)	Public provision
	MKT_PY(i,r)	Output
	MKT_PM(i,r)	Import composite
	MKT_PA(D,i,r)   Armington composite
	MKT_PF(f,r)	Factors
	MKT_PT		Transport services
	MKT_PR(xe,r)	Resources
	MKT_PLS(r)	Labour supply
	MKT_CR(r)	Carbon rights for abating countries
	MKT_PFX		Balance of payment for SOE setting
 
*   Income Balance
	INC_RA(r)	Income balance for representative agent
	INC_GOVT(r)	Income balance for government
 
*   Auxiliary Constraints
	EQU_YIELD(r)	Equal yield constraint;


*	Demand for public output:
DEF_G(i,re)$vgm(i,re)..
	    A_G(i,re) =E=
		    {vgm(i,re) *
		    [ PROD(j$(not fe(j)), (PA("g",j,re)*(1+tg(j,re))/pg0(j,re))**thetag(j,re))*
			  (SUM(fe, vs_fe_e_g(fe,re)*( (PA("g",fe,re)*(1+tg(fe,re))+ co2punit(fe,"govt",re)*pcarb(re))/
				 pg0(fe,re))**(1-eta_g(re))
			     )**(1/(1-eta_g(re)))
			 )**thetag("fe",re)
		    ]*	pg0(i,re) /(PA("g",i,re)*(1+tg(i,re)))
				  }$(not fe(i))
				 +
 
		{vgm(i,re) *
		    [ PROD(j$(not fe(j)), (PA("g",j,re)*(1+tg(j,re))/pg0(j,re))**thetag(j,re))*
			(SUM(fe, vs_fe_e_g(fe,re)*( (PA("g",fe,re)*(1+tg(fe,re))+ co2punit(fe,"govt",re)*pcarb(re))/
				 pg0(fe,re))**(1-eta_g(re))
			     )**(1/(1-eta_g(re)))
			 )**thetag("fe",re)
		    ] /
			SUM(fe, vs_fe_e_g(fe,re)*(((PA("g",fe,re)*(1+tg(fe,re))+
							co2punit(fe,"govt",re)*pcarb(re)))/pg0(fe,re))
							**(1-eta_g(re))
					)**(1/(1-eta_g(re)))
 
			*   [SUM(fe, vs_fe_e_g(fe,re)*(((PA("g",fe,re)*(1+tg(fe,re))+
							co2punit(fe,"govt",re)*pcarb(re)))/pg0(fe,re))
							**(1-eta_g(re))
					)**(1/(1-eta_g(re)))
				*	pg0(i,re)/ (PA("g",i,re)*(1+tg(i,re))+co2punit(i,"govt",re)*pcarb(re) )
		     ]**eta_g(re)
	       }$(fe(i));


*	Private consumption:

DEF_C(i,re)$vpm(i,re)..
       A_C(i,re) =E= 
	    vpm(i,re)*
		   {
		    {[((1-thetac_e(re))*{PROD(j$(not e(j)), (PA("c",j,re)*(1+tp(j,re))
										       /pc0(j,re))**vs_i_ne_c(j,re))
					   }**(1-eta_c(re))
		       +thetac_e(re)*{PROD(e, ((PA("c",e,re)*(1+tp(e,re))
												 +co2punit(e,"final",re)
														   *pcarb(re))
						 /pc0(e,re))**vs_e_e_c(e,re))
								 }**(1-eta_c(re))
		       )**(1/(1-eta_c(re)))
		     ]/
			  (PROD(j$(not e(j)), ((PA("c",j,re)*(1+tp(j,re)))/pc0(j,re))**vs_i_ne_c(j,re)))
					}**eta_c(re)
					*(PROD(j$(not e(j)), ((PA("c",j,re)*(1+tp(j,re)))/pc0(j,re))**vs_i_ne_c(j,re)))
					*    pc0(i,re) / ( PA("c",i,re)*(1+tp(i,re)))
				   }$(not e(i))
   
		+ vpm(i,re)*
 
				{
		    {[((1-thetac_e(re))*{PROD(j$(not e(j)), (PA("c",j,re)*(1+tp(j,re))
										       /pc0(j,re))**vs_i_ne_c(j,re))
					   }**(1-eta_c(re))
		       +thetac_e(re)*{PROD(e, ((PA("c",e,re)*(1+tp(e,re))
												 +co2punit(e,"final",re)
														   *pcarb(re))
						 /pc0(e,re))**vs_e_e_c(e,re))
								 }**(1-eta_c(re))
		       )**(1/(1-eta_c(re)))
		     ]/ PROD(e, ((PA("c",e,re)*(1+tp(e,re))
												 +co2punit(e,"final",re)
														   *pcarb(re))
						 /pc0(e,re))**vs_e_e_c(e,re))
							 }**eta_c(re)
					* PROD(e, ((PA("c",e,re)*(1+tp(e,re))
												 +co2punit(e,"final",re)
														   *pcarb(re))
						 /pc0(e,re))**vs_e_e_c(e,re))
					*    pc0(i,re) / ( PA("c",i,re)*(1+tp(i,re))+ co2punit(i,"final",re)*pcarb(re))
				   }$e(i);
 

*	Demand for domestic input into Armington production:

DEF_A_Y(d,i,re)$(vd(d,i,re)) ..
	 A_A_Y(d,i,re) =E=
		     vd(d,i,re) *
			{[ ((1- thetaa_m(d,i,re)) * py(i,re)**(1-esubdm(i,re))
			    +  thetaa_m(d,i,re)	 *  PM(i,re)**(1-esubdm(i,re))
			    )**(1/(1-esubdm(i,re)))
			 ]/ (py(i,re))
			}**esubdm(i,re);
 
 
* Demand for import composite into Armington production:

DEF_A_M(d,i,re)$(vd(d,i,re)) ..
    A_A_M(d,i,re)$vm(d,i,re) =E= 
	    vm(d,i,re) *
			{[ ((1- thetaa_m(d,i,re)) * py(i,re)**(1-esubdm(i,re))
			    +  thetaa_m(d,i,re)	 * PM(i,re)**(1-esubdm(i,re))
			   )**(1/(1-esubdm(i,re)))
			 ]/ (PM(i,re))
			}**esubdm(i,re);

*	Inter-national transport services (Cobb-Douglas):

DEF_T(i,re)$((vst(i,re)$(not soe)))..
   A_T(i,re) =E=  
	vst(i,re)* [PROD((j,se), py(j,se)**vs_y_t(j,se))]/py(i,re);
 

*	Import

DEF_M_Y(i,se,re)$vxmd(i,se,re)..
    A_M_Y(i,se,re) =E=
		{vxmd(i,se,re)*
	  [  [sum(sse, vs_yt_m(i,sse,re)*
		   [   vs_y_yt(i,sse,re)*py(i,sse)*(1+tm(i,sse,re))*(1+tx(i,sse,re))/pmx0(i,sse,re)
						+ (1-vs_y_yt(i,sse,re))* pt*(1+tm(i,sse,re))/pmt0(i,sse,re)
		   ]**(1-esubmm(i,re))
		  )
			  ]**(1/(1-esubmm(i,re)))
			  / ((vs_y_yt(i,se,re)*py(i,se)*(1+tm(i,se,re))*(1+tx(i,se,re))/pmx0(i,se,re)+
					    (1-vs_y_yt(i,se,re))*pt*(1+tm(i,se,re))/pmt0(i,se,re) ) )
			]**esubmm(i,re)*(1+tm(i,se,re))*(1+tx(i,se,re)) /pmx0(i,se,re)
	 }$(not soe)
	 +
	 {vxmd(i,se,re)*(  [vs_pfx_m(i,re)*(pfx*(1+tm_row(i,re))/pmx0_row(i,re))**(1-esubmm(i,re))
	  + sum(sse, vs_ypfx_m(i,sse,re)*
	      (	 vs_y_ypfx(i,sse,re)*py(i,sse)*(1+tm(i,sse,re))*(1+tx(i,sse,re))/pmx0(i,sse,re)
	      + (1- vs_y_ypfx(i,sse,re))*pfx*(1+tm(i,sse,re))/pmt0(i,sse,re))**(1-esubmm(i,re)))
	    ]**(1/(1-esubmm(i,re)))
	    /(vs_y_ypfx(i,se,re)*py(i,se)*(1+tm(i,se,re))*(1+tx(i,se,re))/pmx0(i,se,re)
	      + (1- vs_y_ypfx(i,se,re))*pfx*(1+tm(i,se,re))/pmt0(i,se,re)))**esubmm(i,re)
		*(1+tm(i,se,re))*(1+tx(i,se,re))/pmx0(i,se,re)
	  }$soe;	


*   We can directly link the demand for PT to A_M_Y because of Leontief structure
 
DEF_M_PFX(i,re)$((mrow(i,re)+ vtwr_row(i,re))$soe)..
    A_M_PFX(i,re) =E=
		(mrow(i,re)+ vtwr_row(i,re))  *
	    [
				[sum(se, vs_ypfx_m(i,se,re)*

		     [vs_y_ypfx(i,se,re)*py(i,se)*(1+tm(i,se,re))*(1+tx(i,se,re))/pmx0(i,se,re)
				     + (1-vs_y_ypfx(i,se,re))*pfx*(1+tm(i,se,re))/pmt0(i,se,re)
		   ]**(1-esubmm(i,re))
		  )
					+ vs_pfx_m(i,re)*(PFX*(1+tm_row(i,re))/pmx0_row(i,re))**(1-esubmm(i,re))
		]**(1/(1-esubmm(i,re)))
			  * pmx0_row(i,re)/(PFX*(1+tm_row(i,re)))
	    ]**esubmm(i,re);
 
*   Note: We account for imports from ROW in the SOE setting directly within the
*	  market clearance condition for PFX (BOP)

 
* Price aggregates in nrp production

* LIQ price aggregate
DEF_P_LIQ(i,re)$((vafm("oil",i,re) + vafm("gas",i,re))$nrp(i)) ..
 A_P_LIQ(i,re) =E=
		   [ v_gas_nrp(i,re)*(( PA("i","gas",re)*(1+ti("gas",i,re))
				       + (PCARB(re)*co2punit("gas",i,re))$abate(re)
				      ) / pa0("gas",i,re)
				     )**(1 - e_g_o(i,re))
		      + (1- v_gas_nrp(i,re))*(( PA("i","oil",re)*(1+ti("oil",i,re))
						+ (PCARB(re)*co2punit("oil",i,re))$abate(re)
					       )/pa0("oil",i,re)
					      )**(1 - e_g_o(i,re))
		   ]**(1/(1 - e_g_o(i,re)));
 

* FF price aggregate

DEF_P_FF(i,re)$((sum(fe,vafm(fe,i,re)))$nrp(i))..
   A_P_FF(i,re) =E=
				[ v_col_nrp(i,re)*(( PA("i","col",re)*(1+ti("col",i,re))
				   + (PCARB(re)*co2punit("col",i,re))$abate(re)
				  ) / pa0("col",i,re)
				 )**(1 - e_c_go(i,re))
		  + (1 - v_col_nrp(i,re))* A_P_LIQ(i,re)**(1 - e_c_go(i,re))
		 ]**(1/(1 - e_c_go(i,re)));
 

* Energy price aggregate

  DEF_P_EN(i,re)$((sum(ec,vafm(ec,i,re)))$nrp(i))..
       A_P_EN(i,re) =E=
    [  v_ele_nrp(i,re)
			* ((PA("i","ele",re)*(1+ti("ele",i,re)))/pa0("ele",i,re))**(1 - e_elec(re))
     + (1 - v_ele_nrp(i,re))
	    * A_P_FF(i,re)**(1 - e_elec(re))
    ]**(1/ (1 - e_elec(re)));
 

* VA price aggregate

DEF_P_VA(i,re)$((sum(f, vfm(f,i,re)))$nrp(i)) ..
	A_P_VA(i,re) =E=
	      PROD(ff, ((PF(ff,re)*(1+tf(ff,i,re)))/pf0(ff,i,re))**v_f_nrp(ff,i,re));
 

* KLE price aggregate

DEF_P_KLE(i,re)$((sum(f, vfm(f,i,re)) + sum(ec,vafm(ec,i,re)))$nrp(i))..
   A_P_KLE(i,re) =E=
   [ v_va_nrp(i,re)* A_P_VA(i,re)**(1-esub(re))
     + (1 - v_va_nrp(i,re)) *	 A_P_EN(i,re)**(1-esub(re))
   ]**(1/(1-esub(re)));
 
 
* Factor demand in non-resource production nrp

DEF_F_NR(f,i,re)$(vfm(f,i,re)$nrp(i))..
    A_F_NR(f,i,re) =E=
   vfm(f,i,re)*
    {A_P_KLE(i,re) / A_P_VA(i,re)}**esub(re)
   * A_P_VA(i,re)
   / (PF(f,re)*(1+tf(f,i,re))/pf0(f,i,re));
 
 
*   Armington good demand in non-resource production

DEF_A_NR(j,i,re)$vafm(j,i,re)..
    A_A_NR(j,i,re) =E=
*	Nonres
  { vafm(j,i,re)
  }$(ne(j)) +
 
*	CRU
  { vafm(j,i,re)
  }$(cru(j)) +
 
*   Ele
  { vafm(j,i,re)*
   {A_P_KLE(i,re) / A_P_EN(i,re)}**esub(re)
   *
   { A_P_EN(i,re) / ((PA("i","ele",re)*(1+ti("ele",i,re)))/pa0("ele",i,re))}**e_elec(re)
  }$(ele(j)) +
 
*   COL
  { vafm(j,i,re)*
   {A_P_KLE(i,re) / A_P_EN(i,re)}**esub(re)
     *
   { A_P_EN(i,re) / A_P_FF(i,re)}**e_elec(re)
   *
   { A_P_FF(i,re) /
		      (( PA("i","col",re)*(1+ti("col",i,re))
			 + (PCARB(re)*co2punit("col",i,re))$abate(re)
			) / pa0("col",i,re)
		       )
   }**e_c_go(i,re)
   * (1+ti("col",i,re)) / pa0("col",i,re)
			    }$(coal(j)) +
 
*   GAS
  { vafm(j,i,re)*
   {A_P_KLE(i,re) / A_P_EN(i,re)}**esub(re)
   *
   { A_P_EN(i,re) / A_P_FF(i,re)}**e_elec(re)
   *
   { A_P_FF(i,re) / A_P_LIQ(i,re)}**e_c_go(i,re)
   *
   { A_P_LIQ(i,re) /
			(( PA("i","gas",re)*(1+ti("gas",i,re))
			   + (PCARB(re)*co2punit("gas",i,re))$abate(re)
			  ) / pa0("gas",i,re)
			 )
   }**e_g_o(i,re)
   *(1+ti("gas",i,re)) / pa0("gas",i,re)
  }$(gas(j)) +
 
*   OIL
  { vafm(j,i,re)*
   {A_P_KLE(i,re) / A_P_EN(i,re)}**esub(re)
   *
   { A_P_EN(i,re) / A_P_FF(i,re)}**e_elec(re)
   *
   { A_P_FF(i,re) / A_P_LIQ(i,re)}**e_c_go(i,re)
   *
   { A_P_LIQ(i,re) /
			(( PA("i","oil",re)*(1+ti("oil",i,re))
			   + (PCARB(re)*co2punit("oil",i,re))$abate(re)
			  ) / pa0("oil",i,re)
			 )
    }**e_g_o(i,re)
    *(1+ti("oil",i,re)) / pa0("oil",i,re)
  }$(oil(j))
 ;
 
 
*   Factor demand in fossil fuel production

DEF_F_R(f,xe,re)$vfm(f,xe,re)..
    A_F_R(f,xe,re) =E=
    vfm(f,xe,re)*
	[
	  [vs_r_xe(xe,re)*(PR(xe,re)/prref(xe,re))**(1-esub_es(xe,re))
	   + (1-vs_r_xe(xe,re))* [  sum(ff, vs_f_xe(ff,xe,re)*(PF(ff,re)*(1+tf(ff,xe,re))/pf0(ff,xe,re)))
				  + sum(j,vs_a_xe(j,xe,re)*
					 (( PA("i",j,re)*(1+ti(j,xe,re))
						+ (PCARB(re)*co2punit(j,xe,re))$abate(re)
					   )/PA0(j,xe,re)
					  )
					)
				 ]**(1-esub_es(xe,re))
	    ]**(1/(1-esub_es(xe,re)))
	    /	    [ sum(ff, vs_f_xe(ff,xe,re)*(PF(ff,re)*(1+tf(ff,xe,re))/pf0(ff,xe,re)))
		      + sum(j,vs_a_xe(j,xe,re)*
			      (( PA("i",j,re)*(1+ti(j,xe,re))
				 + (PCARB(re)*co2punit(j,xe,re))$abate(re)
				)/PA0(j,xe,re)
			       )
			    )
		     ]
	]**esub_es(xe,re)
	 /pf0(f,xe,re)*(1+tf(f,xe,re));
 
 
*   Armington good demand in fossil fuel production

DEF_A_R(j,xe,re)$vafm(j,xe,re)..
    A_A_R(j,xe,re) =E=
    vafm(j,xe,re)*
	[
	  [vs_r_xe(xe,re)*(PR(xe,re)/prref(xe,re))**(1-esub_es(xe,re))
	     +(1-vs_r_xe(xe,re))*[  sum(ff, vs_f_xe(ff,xe,re)*(PF(ff,re)*(1+tf(ff,xe,re))/pf0(ff,xe,re)))
				 + sum(jj,vs_a_xe(jj,xe,re)*
					 (( PA("i",jj,re)*(1+ti(jj,xe,re))
						+ (PCARB(re)*co2punit(jj,xe,re))$abate(re)
					   )/PA0(jj,xe,re)
					  )
					)
				]**(1-esub_es(xe,re))
	      ]**(1/(1-esub_es(xe,re)))
	    /	    [ sum(ff, vs_f_xe(ff,xe,re)*(PF(ff,re)*(1+tf(ff,xe,re))/pf0(ff,xe,re)))
				 + sum(jj,vs_a_xe(jj,xe,re)*
					 (( PA("i",jj,re)*(1+ti(jj,xe,re))
						+ (PCARB(re)*co2punit(jj,xe,re))$abate(re)
					   )/PA0(jj,xe,re)
					  )
					)]
	  ]**esub_es(xe,re)
   * (1+ti(j,xe,re)) / pa0(j,xe,re);
 

*   Fuel-specific resource demand in fossil fuel production

DEF_R_R(xe,re)$rd0(xe,re)..
    A_R_R(xe,re) =E= 
    rd0(xe,re)*
	[
	  [vs_r_xe(xe,re)*(PR(xe,re)/prref(xe,re))**(1-esub_es(xe,re))
	     +(1-vs_r_xe(xe,re))*[  sum(ff, vs_f_xe(ff,xe,re)*(PF(ff,re)*(1+tf(ff,xe,re))/pf0(ff,xe,re)))
				 + sum(j,vs_a_xe(j,xe,re)*
					 (( PA("i",j,re)*(1+ti(j,xe,re))
						+ (PCARB(re)*co2punit(j,xe,re))$abate(re)
					   )/PA0(j,xe,re)
					  )
					)
				]**(1-esub_es(xe,re))
	    ]**(1/(1-esub_es(xe,re)))
			* prref(xe,re)
			/ PR(xe,re)
	  ]**esub_es(xe,re);
 
 
*	Supply of production to domestic market (CET)

DEF_SD_Y(i,re)$(vom(i,re)$soe)..
    A_SD_Y(i,re) =E= 
	    vom(i,re) *
						(py(i,re)
						 / ( thetad(i,re)  * py(i,re)**(1+etrn(re)) +
			     (1-thetad(i,re)) * PFX**(1+etrn(re))
			    )**(1/(1+etrn(re)))
			)**(etrn(re));
 
   
*	Supply of production to ROW (PFX)

DEF_SX_Y(i,re)$(xrow(i,re)$soe)..
    A_SX_Y(i,re) =E= 

		xrow(i,re) *
			    (PFX / 
			    ( thetad(i,re)  * py(i,re)**(1+etrn(re)) +
			      (1-thetad(i,re)) * PFX**(1+etrn(re))
			    )**(1/(1+etrn(re)))
			 )**(etrn(re));


*   Zero profit per unit activity are defined here:
*   ==============================================
 
*   Non-resource Production:
 
PRF_Y(i,re)$(vom(i,re)>0)..
      [	  SUM(j, A_A_NR(j,i,re)*( PA("i",j,re) * (1+ti(j,i,re)) + (co2punit(j,i,re)*PCARB(re))$abate(re))
	      )
	+ SUM(f, A_F_NR(f,i,re) * PF(f,re) * (1 + tf(f,i,re)) )
      ]$((vom(i,re)>0)$nrp(i))
	+
	  [  SUM(j, A_A_R(j,i,re)*( PA("i",j,re) * (1+ti(j,i,re)) + (co2punit(j,i,re)*PCARB(re))$abate(re))
	    )
       + SUM(f, A_F_R(f,i,re) * PF(f,re) * (1 + tf(f,i,re)) )
       + A_R_R(i,re)*PR(i,re)
      ]$((vom(i,re)>0)$xe(i))
      =E=
      {	  [	     ( (1 - ty(i,re)) * PY(i,re) * vom(i,re))$(not soe)
	 +  ((1 - ty(i,re))*(PY(i,re)*A_SD_Y(i,re) + PFX*A_SX_Y(i,re)))$SOE
	 ]$((vom(i,re)>0)$nrp(i))
       +
	[	    ((1 - ty(i,re)) * PY(i,re) * vom(i,re))$(not soe)
	 +  ((1 - ty(i,re))*(PY(i,re)*A_SD_Y(i,re) + PFX*A_SX_Y(i,re)))$SOE
	]$((vom(i,re)>0)$xe(i))
	};

 
*   Armington Production

PRF_A(d,i,re)$va(d,i,re)..
	A_A_M(d,i,re)*PM(i,re) + A_A_Y(d,i,re)*PY(i,re) =E= PA(d,i,re)*va(d,i,re);

 
 
PRF_M(i,re)$vim(i,re)..
    SUM(se, (1 + tm(i,se,re)) * A_M_Y(i,se,re) *(    PY(i,se) * (1 + tx(i,se,re))+ adj_base(i,se,re)*PT ) )$(not soe)
	+ { sum(se, (1 + tm(i,se,re)) * A_M_Y(i,se,re) * (PY(i,se) * (1 + tx(i,se,re)) + adj_base(i,se,re)*PFX))
	    + (1 + tm_row(i,re)) * A_M_PFX(i,re)*PFX}$SOE
	=E= PM(i,re) * vim(i,re);
 
 
*   Public output:
 
PRF_G(re)..  SUM(i, (PA("g",i,re) * (1+tg(i,re)) + PCARB(re)*CO2PUNIT(I,"govt",re)$ABATE(RE)) * A_G(i,re))
				    =E= PG(re) * vg(re);

 
*   Private consumption:
 
PRF_C(re)..  SUM(i, (PA("c",i,re) * (1+tp(i,re))+PCARB(re)*CO2PUNIT(I,"final",re)$ABATE(RE) )  * A_C(i,re))
					=E= PC(re) * vp(re);
 
 
*   Inter-national transport services (Cobb-Douglas):
 
PRF_YT$(NOT SOE)..  PROD((i,re), PY(i,re)**thetat(i,re)) =E= PT;
 
 
*   Labor supply
PRF_LS(RE)..	    PLS(re)*(1+tau(re))	 =E= PF("LAB",RE);
 

*   Market clearing
*   ===============
 
*   Aggregate output (PY):
 
MKT_PY(i,re)..
    (vom(i,re) * Y(i,re))$(not soe) + (A_SD_Y(i,re)*Y(i,re))$soe
	=E= {SUM(d, A_A_Y(d,i,re)*A(d,i,re)) +	SUM(se, A_M_Y(i,re,se)*M(i,se)) + (vi(re))$cgd(i)
			+ (A_T(i,re)*YT)$(not soe)
	  }		;
 
 
*   Imports (PM):
 
MKT_PM(i,re)$vim(i,re)..   vim(i,re) * M(i,re) =E= sum(d, A_A_M(d,i,re)*A(d,i,re));
 
   
*   International transport (PT):
 
MKT_PT$(not soe)..   YT * vt =E=  sum((i,re,se), A_M_Y(i,re,se) * M(i,se) * tpunit(i,re,se));
 
 
*   Armington (PA):
 
MKT_PA(d,i,re)$va(d,i,re)..
    va(d,i,re) * A(d,i,re) =E= 
	  {  [sum(j$nrp(j), A_A_NR(i,j,re)*Y(j,re)) + sum(j$xe(j), A_A_R(i,j,re)*Y(j,re))]$i_d(d)
	    + (A_C(i,re) * C(re))$c_d(d) + ( A_G(i,re) *  G(re))$g_d(d)
	  };
 
 
*   Factor market (PF):

MKT_PF(f,re)..	 evoa(f,re)$(capital(f)) + LS(re)*evoa("LAB",re)$(not capital(f))
			=E= {sum(i$nrp(i), A_F_NR(f,i,re) * Y(i,re)) + sum(i$xe(i), A_F_R(f,i,re) * Y(i,re))
			  } ;
 
 
*   Resource market (PR):

MKT_PR(xe,re)..	     rd0(xe,re)*ffsupply(xe,re) =E= A_R_R(xe,re)*Y(xe,re);
 
 
*   Labour supply (PLS):

MKT_PLS(RE)..	    evoa("LAB",re) =E= LS(re)*evoa("LAB",re);
 
 
*   Carbon rights (PCARB):

MKT_CR(RE)$abate(re)..	    carblim(re)$abate(re) 
			    =E= {
				    sum(i$nrp(i), sum(fe, A_A_NR(fe,i,re)*Y(i,re)*co2punit(fe,i,re)))
				  + sum(xe, sum(fe$vafm(fe,xe,re), A_A_R(fe,xe,re)*Y(xe,re)*co2punit(fe,xe,re)))
				  + sum(fe, A_C(fe,re) * C(re)*co2punit(fe,"final",re))
				  + sum(fe, A_G(fe,re) *  G(re)*co2punit(fe,"govt",re))
				 }			    ;
 

*   Private demand:
 
MKT_PC(re)..  C(re) * vp(re) =E= RA(re) / PC(re) ;
 
 
*   Government provision:
 
MKT_PG(re).. G(re)*vg(re) =E= GOVT(re)/PG(re);
   
 
*   BOP for SOE setting
 
MKT_PFX$SOE..	    sum((i,re), A_SX_Y(i,re)*Y(i,re)*PFX) + sum(re, tax_row(re)*PFX + vb(re)*PFX)
			     =E=  { sum((i,re), A_M_PFX(i,re)*M(i,re)*PFX) + sum((i,se,re), A_M_Y(i,se,re)*tpunit(i,se,re)*M(i,re)*PFX)};

 
set num_re(re);
num_re(re) = yes$num(re);
display num_re, num;

 
*   Income balance
*   ==============
 
INC_RA(re)..

    RA(re) =E= sum(i, PR(i,re)*(rd0(i,re) *ffsupply(i, re)))
				+ PF("cap",re)*evoa("cap",re)
		+ PLS(re)*evoa("lab",re)
		+ sum(num_re, PC(num_re) * vb(re))$(not soe) + (PFX*vb(re))$soe
		- sum(cgd, PY(cgd,re)* vi(re))
		- PC(re) * transfer(re);

 
INC_GOVT(re)..	

    GOVT(re) =E=   PC(re) * transfer(re)
		+ (PCARB(re)*carblim(re))$abate(re)
		+ (PFX*tax_row(re))$soe
* Taxes on goverment consumption:
		+    sum(i, tg(i,re) * PA("g",i,re) * A_G(i,re) * G(re))
* Taxes on private consumption:
		+    sum(i, tp(i,re) * PA("c",i,re) * A_C(i,re) * C(re))
* Taxes on factor use:
    +	 sum( (i,f)$nrp(i),  tf(f,i,re) * PF(f,re) * A_F_NR(f,i,re) * Y(i,re))
    +	 sum( (i,f)$xe(i),  tf(f,i,re) * PF(f,re) * A_F_R(f,i,re) * Y(i,re))
    +	 LS(re)*evoa("LAB",re)*PLS(re)*tau(re)
* Output tax:
    +	 sum(i, ty(i,re) *  (A_SD_Y(i,re)* Y(i,re)*PY(i,re) + A_SX_Y(i,re)* Y(i,re)*PFX))$soe
    +	 sum(i, ty(i,re) *  Y(i,re)*VOM(i,re)*PY(i,re))$(not soe)
* Tax on intermediate demand:
    +	 sum((i,j)$nrp(i), ti(j,i,re) * PA("i",j,re) *A_A_NR(j,i,re)* Y(i,re))
    +	 sum((i,j)$xe(i), ti(j,i,re) * PA("i",j,re) *A_A_R(j,i,re)* Y(i,re))
* Export tax:
    +	  sum((i,se), tx(i,re,se) * PY(i,re)*A_M_Y(i,re,se)*M(i,se))
* Import tariff applies to mechandise gross of export tax
* and transport cost:
    +	 sum((i,se), tm(i,se,re) * A_M_Y(i,se,re)*M(i,re) *
			(PY(i,se) * (1+tx(i,se,re)) + (PT * tpunit(i,se,re))$(NOT SOE)
								    + (PFX * tpunit(i,se,re))$SOE))
    +	(sum(i, tm_row(i,re)* A_M_PFX(i,re)*M(i,re) *PFX))$soe;

 
*   Auxiliary Constraints
*   =====================

EQU_YIELD(re)..	   G(re) =E= GSIZE(re);

*   Define the mcp model:
*   ===================== 

MODEL GTAP_MCP /
	DEF_G.A_G
	DEF_C.A_C
	DEF_A_Y.A_A_Y
	DEF_A_M.A_A_M
	DEF_T.A_T
	DEF_M_Y.A_M_Y
	DEF_M_PFX.A_M_PFX
	DEF_F_NR.A_F_NR
	DEF_F_R.A_F_R
	DEF_A_NR.A_A_NR
	DEF_A_R.A_A_R
	DEF_R_R.A_R_R
	DEF_SD_Y.A_SD_Y
	DEF_SX_Y.A_SX_Y
	DEF_P_LIQ.A_P_LIQ
	DEF_P_FF.A_P_FF
	DEF_P_EN.A_P_EN
	DEF_P_VA.A_P_VA
	DEF_P_KLE.A_P_KLE
	PRF_C.C
	PRF_G.G
	PRF_Y.Y
	PRF_M.M
	PRF_A.A
	PRF_YT.YT
	PRF_LS.LS
	MKT_PC.PC
	MKT_PG.PG
	MKT_PY.PY
	MKT_PM.PM
	MKT_PA.PA
	MKT_PF.PF
	MKT_PT.PT
	MKT_PR.PR
	MKT_PLS.PLS
	MKT_CR.PCARB
	MKT_PFX.PFX
	INC_RA.RA
	INC_GOVT.GOVT
	EQU_YIELD.TAU
    /;
 
 
* Install lower bounds to avoid bad function calls:

PC.LO(re)	=1.e-5;
PG.LO(re)	=1.e-5; 
PY.LO(i,re)	=1.e-5;
PM.LO(i,re)	=1.e-5;
PA.LO(d,i,re)	=1.e-5;
PF.LO(f,re)	=1.e-5; 
PT.LO		=1.e-5;
PLS.LO(re)	=1.e-5;
PR.LO(i,re)	=1.e-5;
PFX.LO		=1.e-5;
A_P_KLE.LO(i,re)=1.e-5;
A_P_LIQ.LO(i,re)=1.e-5;
A_P_FF.LO(i,re) =1.e-5;
A_P_EN.LO(i,re) =1.e-5;
A_P_VA.LO(i,re) =1.e-5;
 

* Install level values:

C.L(re)		=1;
G.L(re)		=1;
Y.L(i,re)	=1;
M.L(i,re)	=1;
A.L(d,i,re)	=1;
YT.L		=1;
LS.L(re)	=1;
PC.L(re)	=1;
PG.L(re)	=1;
PY.L(i,re)	=1;
PM.L(i,re)	=1;
PA.L(d,i,re)	=1;
PF.L(f,re)	=1;
PT.L		=1;
PLS.L(re)	=1;
PR.L(i,re)	=1;
PFX.L		=1;
PCARB.L(re)	=0;
RA.L(re)=vp(re);
GOVT.L(re)=vg(re);
 
*   Fix a numeraire to simplify comparison with MGE:
RA.FX(num_re) = RA.L(num_re);

A_G.FX(i,re)$(not vgm(i,re))		    = 0;
A_C.FX(i,re)$(not vpm(i,re))		    = 0;
A_A_Y.FX(d,i,re)$(not vd(d,i,re))	    = 0;
A_A_M.FX(d,i,re)$(not vm(d,i,re))	    = 0;
A_M_Y.FX(i,se,re)$(not vxmd(i,se,re))	    = 0;
A_T.FX(i,re)$(not vst(i,re))		    = 0;
A_M_PFX.FX(i,re)$(not (mrow(i,re)+vtwr_row(i,re))) = 0;
A_F_NR.FX(f,i,re)$((not vfm(f,i,re))$nrp(i))= 0;
A_F_R.FX(f,xe,re)$(not vfm(f,xe,re))	    = 0;
A_A_NR.FX(j,i,re)$((not vafm(j,i,re))$nrp(i))= 0;
A_A_R.FX(j,i,re)$(not vafm(j,i,re))	    = 0;
A_A_R.FX(j,i,re)$(not xe(i))		    = 0;
A_R_R.FX(i,re)$(not rd0(i,re))		    = 0;
A_SD_Y.FX(i,re)$(not vom(i,re))		    = 0;
A_SX_Y.FX(i,re)$(not xrow(i,re))	    = 0;
A_P_KLE.FX(i,re)$((not (sum(f, vfm(f,i,re)) + sum(ec,vafm(ec,i,re)))) or (not nrp(i)))	= 0;
A_P_LIQ.FX(i,re)$((not (vafm("oil",i,re) + vafm("gas",i,re)))or (not nrp(i))) 		= 0;
A_P_FF.FX(i,re)$((not (sum(fe,vafm(fe,i,re)))) or (not nrp(i)))				= 0;
A_P_EN.FX(i,re)$((not(sum(ec,vafm(ec,i,re)))) or (not nrp(i)))				= 0;
A_P_VA.FX(i,re)$((not (sum(f, vfm(f,i,re)))) or (not nrp(i)))				= 0;

Y.FX(i,re)$(not vom(i,re))		    = 0;
M.FX(i,re)$(not vim(i,re))		    = 0;
A.FX(d,i,re)$(not va(d,i,re))		    = 0;
 
PA.FX(d,i,re)$(not va(d,i,re))		    = 0.0001;
PM.FX(i,re)$(not vim(i,re))		    = 0.0001;
PY.FX(i,re)$(not vom(i,re))		    = 0.0001;
 
A_G.L(i,re)		= vgm(i,re);
A_C.L(i,re)		= vpm(i,re);
A_A_Y.L(d,i,re)		= vd(d,i,re);
A_A_M.L(d,i,re)		= vm(d,i,re);
A_M_Y.L(i,se,re)	= vxmd(i,se,re);
A_T.L(i,re)		= vst(i,re);
A_M_PFX.L(i,re)		=(mrow(i,re)+vtwr_row(i,re));
A_F_NR.L(f,i,re)$nrp(i)	= vfm(f,i,re);
A_F_R.L(f,xe,re)	= vfm(f,xe,re);
A_A_NR.L(j,i,re)$nrp(i)	= vafm(j,i,re);
A_A_R.L(j,xe,re)	= vafm(j,xe,re);
A_R_R.L(i,re)		= rd0(i,re);
A_SD_Y.L(i,re)		= vom(i,re);
A_SX_Y.L(i,re)		= xrow(i,re);
 
A_P_KLE.L(i,re)		= 1;
A_P_LIQ.L(i,re)		= 1;
A_P_FF.L(i,re)		= 1;
A_P_EN.L(i,re)		= 1;
A_P_VA.L(i,re)		= 1;
 
 

*   Replicate the benchmark
*   =======================

PCARB.FX(re)$(not abate(re)) = 0;
gtap_mcp.ITERLIM = 0;
SOLVE gtap_mcp USING MCP;
ABORT$(ABS(gtap_mcp.OBJVAL) gt 1e-4) "Model does not calibrate", gtap_mcp.OBJVAL;
DISPLAY "The following reports precision of the benchmark dataset:", gtap_mcp.OBJVAL;

$exit
*   Do the tax counterfactual - some illustrative carbon abatement scenario
*   =========================================================================

PARAMETER REPORT	Results reporting;
    
gtap.ITERLIM	= 8000;
abate(annexb)	= yes;
carblim(annexb)	= 0.95 *carblim(annexb);
TAU.UP(re)	= +INF;
TAU.LO(re)	= -1;

*	Relax carbon price for all abating countries
PCARB.LO(re)$(abate(re))	= 0;
PCARB.UP(re)$(abate(re))	= +INF;

$INCLUDE GTAP.GEN
SOLVE GTAP USING MCP;

report(re,"%_EV","MPSGE") = ROUND(100*(C.L(re) -1), 2);


*   Check equivalence of MCP model with MPSGE model
*   ===============================================
gtap_mcp.WORKSPACE  = 100;
gtap_mcp.ITERLIM    = 8000;
SOLVE gtap_mcp USING MCP;

report(re,"%_EV","MCP") = ROUND(100*(C.L(re) -1), 2);


DISPLAY report;